// ChooseTile.cpp : implementation file
//

#include "stdafx.h"
#include "OpenGLApp.h"
#include "ChooseTile.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CChooseTile dialog


CChooseTile::CChooseTile(CWnd* pParent /*=NULL*/)
	: CDialog(CChooseTile::IDD, pParent)
{
	//{{AFX_DATA_INIT(CChooseTile)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	imageDesc = NULL;
}


void CChooseTile::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CChooseTile)
	DDX_Control(pDX, IDC_DIFFTILES, m_cTabTiles);
	DDX_Control(pDX, IDC_TILES, m_cTilesThumbs);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CChooseTile, CDialog)
	//{{AFX_MSG_MAP(CChooseTile)
	ON_NOTIFY(TCN_SELCHANGE, IDC_DIFFTILES, OnSelectDifferentSet)
	ON_WM_DESTROY()
	ON_NOTIFY(NM_CLICK, IDC_TILES, OnClickTiles)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CChooseTile message handlers

BOOL CChooseTile::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	m_cTabTiles.InsertItem(0, _T("Ground Tiles"));
	m_cTabTiles.InsertItem(1, _T("Items"));
	m_cTabTiles.InsertItem(2, _T("Special Items"));
	m_cTabTiles.InsertItem(3, _T("Enemies"));
	

	DWORD dwStyle = m_cTilesThumbs.GetExtendedStyle();
	dwStyle |= LVS_EX_FULLROWSELECT;
	m_cTilesThumbs.SetExtendedStyle(dwStyle);

	LoadTilesets("tilesets.txt");
	m_cTabTiles.SetCurSel(0);
	OnSelectDifferentSet(NULL, NULL);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CChooseTile::OnSelectDifferentSet(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// Add selected tileset tile images
	m_cTilesThumbs.DeleteAllItems();

	if(m_cTabTiles.GetCurSel() <= m_iTilesets)
	{
		m_cTilesThumbs.SetImageList(&m_cTileset[m_cTabTiles.GetCurSel()], LVSIL_NORMAL);

		for (int i=0; i<m_iTilesCount[m_cTabTiles.GetCurSel()]; i++)
		{
			// Add items
			LVITEM lvi;
			CString strItem;
			// Insert the first item
			lvi.mask =  LVIF_IMAGE | LVIF_TEXT;

			int ttl = 0;
			if(m_cTabTiles.GetCurSel() > 0)
			{
				for (int g=0; g<m_cTabTiles.GetCurSel(); g++)
					ttl+= m_iTilesCount[g];
			}
			if(imageDesc !=NULL)
			{
					strItem = CString(imageDesc[ttl + i]);
					strItem.Remove('\n');
			}

			lvi.iItem = i;
			lvi.iSubItem = 0;
			lvi.pszText = (LPTSTR)(LPCTSTR)(strItem);
			lvi.iImage = i%m_iTilesCount[m_cTabTiles.GetCurSel()];		// There are 8 images in the image list
			m_cTilesThumbs.InsertItem(&lvi);
		}
	}
//	*pResult = 0;
}

BOOL CChooseTile::LoadTilesets(char *filen)
{
	char entry[1024] = "";
	char filename[255]= "";
	int numinset = 0;
	int r, g, b, imgsize;
	
	FILE *file = fopen(filen, "r");

	if(file == NULL)
		return false;
	
	fgets(entry, 255, file);
	sscanf(entry, "%d\n", &m_iTilesets);

	
	if(m_iTilesets > 0)
	{
		m_cTileset = new CImageList[m_iTilesets];
		m_iTilesCount = new int[m_iTilesets];

		total_images = 0;
		for (int i=0; i<m_iTilesets; i++)
		{
			fgets(entry, 255, file);
			sscanf(entry, "%s %d %d %d %d %d\n", filename, &numinset, &imgsize, &r, &g, &b);
		
			HIMAGELIST hList = ImageList_Create(imgsize, imgsize, ILC_COLOR8 |ILC_MASK , 1, 1);
			m_cTileset[i].Attach(hList);
			
			total_images+=numinset;

			CBitmap cBmp;
			LoadBMPImage(filename, cBmp, NULL);

			// Load images from bitmap file
			m_cTileset[i].Add(&cBmp, RGB(r, g, b));
			cBmp.DeleteObject();

			m_iTilesCount[i] = numinset;
		}

		imageDesc = new char*[total_images];
		for (int r=0; r<total_images; r++)
			imageDesc[r] = new char[32];
			
		for (r=0; r<total_images; r++)
			fgets(imageDesc[r], 32, file);
		
		fclose(file);
		return true;
	}

return false;
}

// LoadBMPImage- Loads a BMP file and creates a bitmap GDI object
//  also creates logical palette for it.
// Returns- TRUE for success
// sBMPFile- Full path of the BMP file
// bitmap- The bitmap object to initialize
// pPal- Will hold the logical palette. Can be NULL
BOOL CChooseTile::LoadBMPImage( LPCTSTR sBMPFile, CBitmap& bitmap, CPalette *pPal )
{
	CFile file;
	if( !file.Open( sBMPFile, CFile::modeRead) )
		return FALSE;

	BITMAPFILEHEADER bmfHeader;

	// Read file header
	if (file.Read((LPSTR)&bmfHeader, sizeof(bmfHeader)) != sizeof(bmfHeader))
		return FALSE;

	// File type should be 'BM'
	if (bmfHeader.bfType != ((WORD) ('M' << 8) | 'B'))
		return FALSE;

	// Get length of the remainder of the file and allocate memory
	DWORD nPackedDIBLen = file.GetLength() - sizeof(BITMAPFILEHEADER);
	HGLOBAL hDIB = ::GlobalAlloc(GMEM_FIXED, nPackedDIBLen);
	if (hDIB == 0)
		return FALSE;

	// Read the remainder of the bitmap file.
	if (file.ReadHuge((LPSTR)hDIB, nPackedDIBLen) != nPackedDIBLen )
	{
		GlobalFree(hDIB);
		return FALSE;
	}


	BITMAPINFOHEADER &bmiHeader = *(LPBITMAPINFOHEADER)hDIB ;
	BITMAPINFO &bmInfo = *(LPBITMAPINFO)hDIB ;

	// If bmiHeader.biClrUsed is zero we have to infer the number
	// of colors from the number of bits used to specify it.
	int nColors = bmiHeader.biClrUsed ? bmiHeader.biClrUsed :
	1 << bmiHeader.biBitCount;

	LPVOID lpDIBBits;
	if( bmInfo.bmiHeader.biBitCount > 8 )
		lpDIBBits = (LPVOID)((LPDWORD)(bmInfo.bmiColors + bmInfo.bmiHeader.biClrUsed) +
		((bmInfo.bmiHeader.biCompression == BI_BITFIELDS) ? 3 : 0));
	else
		lpDIBBits = (LPVOID)(bmInfo.bmiColors + nColors);

	// Create the logical palette
	if( pPal != NULL )
	{
	// Create the palette
		if( nColors <= 256 )
		{
			UINT nSize = sizeof(LOGPALETTE) + (sizeof(PALETTEENTRY) * nColors);
			LOGPALETTE *pLP = (LOGPALETTE *) new BYTE[nSize];

			pLP->palVersion = 0x300;
			pLP->palNumEntries = nColors;

			for( int i=0; i < nColors; i++)
			{
				pLP->palPalEntry[i].peRed = bmInfo.bmiColors[i].rgbRed;
				pLP->palPalEntry[i].peGreen = bmInfo.bmiColors[i].rgbGreen;
				pLP->palPalEntry[i].peBlue = bmInfo.bmiColors[i].rgbBlue;
				pLP->palPalEntry[i].peFlags = 0;
			}

			pPal->CreatePalette( pLP );

			delete[] pLP;
		}
	}

	CClientDC dc(NULL);
	CPalette* pOldPalette = NULL;
	if( pPal )
	{
		pOldPalette = dc.SelectPalette( pPal, FALSE );
		dc.RealizePalette();
	}

	HBITMAP hBmp = CreateDIBitmap( dc.m_hDC,// handle to device context
	&bmiHeader,// pointer to bitmap size and format data
	CBM_INIT,// initialization flag
	lpDIBBits,// pointer to initialization data
	&bmInfo,// pointer to bitmap color-format data
	DIB_RGB_COLORS);// color-data usage
	bitmap.Attach( hBmp );

	if( pOldPalette )
		dc.SelectPalette( pOldPalette, FALSE );

	GlobalFree(hDIB);
	return TRUE;
}

void CChooseTile::OnDestroy() 
{
	CDialog::OnDestroy();
	
	SAFEDEL(m_cTileset);
	SAFEDEL(m_iTilesCount);

	for (int r=0; r<total_images; r++)
		delete imageDesc[r];

	SAFEDEL(imageDesc);


}

void CChooseTile::OnClickTiles(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	
	int iSelectedTile = m_cTilesThumbs.GetSelectionMark();

	// Send message saying what tileset and what item has been selected
	pParent->PostMessage(MSG_SELTILE, MAKEWPARAM(iSelectedTile, m_cTabTiles.GetCurSel()), 0);
	*pResult = 0;
}
