#include "StdAfx.h"
#include "Maths.h"

//----------------------------------------------------------------------------
double FastSin0 (double fAngle)
{
    double fASqr = fAngle*fAngle;
    double fResult = 7.61e-03f;
    fResult *= fASqr;
    fResult -= 1.6605e-01f;
    fResult *= fASqr;
    fResult += 1.0f;
    fResult *= fAngle;
    return fResult;
}
//----------------------------------------------------------------------------
double FastSin1 (double fAngle)
{
    double fASqr = fAngle*fAngle;
    double fResult = -2.39e-08f;
    fResult *= fASqr;
    fResult += 2.7526e-06f;
    fResult *= fASqr;
    fResult -= 1.98409e-04f;
    fResult *= fASqr;
    fResult += 8.3333315e-03f;
    fResult *= fASqr;
    fResult -= 1.666666664e-01f;
    fResult *= fASqr;
    fResult += 1.0f;
    fResult *= fAngle;
    return fResult;
}
//----------------------------------------------------------------------------
double FastCos0 (double fAngle)
{
    double fASqr = fAngle*fAngle;
    double fResult = 3.705e-02f;
    fResult *= fASqr;
    fResult -= 4.967e-01f;
    fResult *= fASqr;
    fResult += 1.0f;
    return fResult;
}
//----------------------------------------------------------------------------
double FastCos1 (double fAngle)
{
    double fASqr = fAngle*fAngle;
    double fResult = -2.605e-07f;
    fResult *= fASqr;
    fResult += 2.47609e-05f;
    fResult *= fASqr;
    fResult -= 1.3888397e-03f;
    fResult *= fASqr;
    fResult += 4.16666418e-02f;
    fResult *= fASqr;
    fResult -= 4.999999963e-01f;
    fResult *= fASqr;
    fResult += 1.0f;
    return fResult;
}
//----------------------------------------------------------------------------
double FastTan0 (double fAngle)
{
    double fASqr = fAngle*fAngle;
    double fResult = 2.033e-01f;
    fResult *= fASqr;
    fResult += 3.1755e-01f;
    fResult *= fASqr;
    fResult += 1.0f;
    fResult *= fAngle;
    return fResult;
}
//----------------------------------------------------------------------------
double FastTan1 (double fAngle)
{
    double fASqr = fAngle*fAngle;
    double fResult = 9.5168091e-03f;
    fResult *= fASqr;
    fResult += 2.900525e-03f;
    fResult *= fASqr;
    fResult += 2.45650893e-02f;
    fResult *= fASqr;
    fResult += 5.33740603e-02f;
    fResult *= fASqr;
    fResult += 1.333923995e-01f;
    fResult *= fASqr;
    fResult += 3.333314036e-01f;
    fResult *= fASqr;
    fResult += 1.0f;
    fResult *= fAngle;
    return fResult;
}
//----------------------------------------------------------------------------
double FastInvSin (double fValue)
{
    double fRoot = sqrt(1.0f-fValue);
    double fResult = -0.0187293f;
    fResult *= fValue;
    fResult += 0.0742610f;
    fResult *= fValue;
    fResult -= 0.2121144f;
    fResult *= fValue;
    fResult += 1.5707288f;
    fResult = PI/2 - fRoot*fResult;
    return fResult;
}
//----------------------------------------------------------------------------
double FastInvCos (double fValue)
{
    double fRoot = sqrt(1.0f-fValue);
    double fResult = -0.0187293f;
    fResult *= fValue;
    fResult += 0.0742610f;
    fResult *= fValue;
    fResult -= 0.2121144f;
    fResult *= fValue;
    fResult += 1.5707288f;
    fResult *= fRoot;
    return fResult;
}
//----------------------------------------------------------------------------
double FastInvTan0 (double fValue)
{
    double fVSqr = fValue*fValue;
    double fResult = 0.0208351f;
    fResult *= fVSqr;
    fResult -= 0.085133f;
    fResult *= fVSqr;
    fResult += 0.180141f;
    fResult *= fVSqr;
    fResult -= 0.3302995f;
    fResult *= fVSqr;
    fResult += 0.999866f;
    fResult *= fValue;
    return fResult;
}
//----------------------------------------------------------------------------
double FastInvTan1 (double fValue)
{
    double fVSqr = fValue*fValue;
    double fResult = 0.0028662257f;
    fResult *= fVSqr;
    fResult -= 0.0161657367f;
    fResult *= fVSqr;
    fResult += 0.0429096138f;
    fResult *= fVSqr;
    fResult -= 0.0752896400f;
    fResult *= fVSqr;
    fResult += 0.1065626393f;
    fResult *= fVSqr;
    fResult -= 0.1420889944f;
    fResult *= fVSqr;
    fResult += 0.1999355085f;
    fResult *= fVSqr;
    fResult -= 0.3333314528f;
    fResult *= fVSqr;
    fResult += 1.0f;
    fResult *= fValue;
    return fResult;
}

/* --------------------------------- VECTOR IMPLEMENTATION -------------------------------//
//----------------------------------------------------------------------------------------*/
const CVector& CVector::operator = (const CVector& V)
{
	x = V.x;
	y = V.y;
	z = V.z;
		
	return *this;
}

const CVector& CVector::operator = (const float * d)
{
	x = d[0];
	y = d[1];
	z = d[2];
		
	return *this;
}

void CVector::Transform(CMatrix *m)
{
	double vector[4];
	const float *matrix = m->m_matrix;

	vector[0] = m_pos[0]*matrix[0]+m_pos[1]*matrix[4]+m_pos[2]*matrix[8]+matrix[12];
	vector[1] = m_pos[0]*matrix[1]+m_pos[1]*matrix[5]+m_pos[2]*matrix[9]+matrix[13];
	vector[2] = m_pos[0]*matrix[2]+m_pos[1]*matrix[6]+m_pos[2]*matrix[10]+matrix[14];
	vector[3] = m_pos[0]*matrix[3]+m_pos[1]*matrix[7]+m_pos[2]*matrix[11]+matrix[15];

	m_pos[0] = ( float )( vector[0] );
	m_pos[1] = ( float )( vector[1] );
	m_pos[2] = ( float )( vector[2] );
	m_pos[3] = ( float )( vector[3] );
}

void CVector::Transform3(CMatrix *m)
{
	double vector[3];
	const float *matrix = m->m_matrix;

	vector[0] = m_pos[0]*matrix[0]+m_pos[1]*matrix[4]+m_pos[2]*matrix[8];
	vector[1] = m_pos[0]*matrix[1]+m_pos[1]*matrix[5]+m_pos[2]*matrix[9];
	vector[2] = m_pos[0]*matrix[2]+m_pos[1]*matrix[6]+m_pos[2]*matrix[10];

	m_pos[0] = ( float )( vector[0] );
	m_pos[1] = ( float )( vector[1] );
	m_pos[2] = ( float )( vector[2] );
	m_pos[3] = 1;
}

	//	Add & Substract, Positive & Negative
	const CVector CVector::operator + (const CVector& Vector) const
	{	return CVector(x + Vector.x, y + Vector.y, z + Vector.z);
	}
	const CVector CVector::operator + () const
	{	return CVector(*this);
	}
	const CVector& CVector::operator += (const CVector& Vector)
	{	x += Vector.x;
		y += Vector.y;
		z += Vector.z;
		
		return *this;
	}
	// minus vector 
	const CVector CVector::operator - (const CVector& Vector) const
	{	return CVector(x - Vector.x, y - Vector.y, z - Vector.z);
	}
	const CVector CVector::operator - () const
	{	return CVector(-x, -y, -z);
	}
	const CVector &CVector::operator -= (const CVector& Vector)
	{	x -= Vector.x;
		y -= Vector.y;
		z -= Vector.z;
		
		return *this;
	}

	//	Multiply & Divide
	const CVector CVector::operator * (const CVector& Vector) const
	{	return CVector(x * Vector.x, y * Vector.y, z * Vector.z);
	}
	const CVector &CVector::operator *= (const CVector& Vector)
	{	x *= Vector.x;
		y *= Vector.y;
		z *= Vector.z;
		
		return *this;
	}
	const CVector CVector::operator * (const float d) const
	{	return CVector(x * d, y * d, z * d);
	}
	const CVector &CVector::operator *= (const float d)
	{	x *= d;
		y *= d;
		z *= d;
		
		return *this;
	}
	const CVector CVector::operator / (const CVector& Vector) const
	{	return CVector(x / Vector.x, y / Vector.y, z / Vector.z);
	}
	
	const CVector &CVector::operator /= (const CVector& Vector)
	{	x /= Vector.x;
		y /= Vector.y;
		z /= Vector.z;
		
		return *this;
	}
	const CVector CVector::operator / (const float d) const
	{	const float _d = 1.0f / d;
		return CVector(x * _d, y * _d, z * _d);
	}
	const CVector &CVector::operator /= (const float d)
	{	const float _d = 1.0f / d;
		x *= _d;
		y *= _d;
		z *= _d;
		
		return *this;
	}
	
	//	Dot Product
	const float CVector::Dot(const CVector& Vector) const
	{	return x*Vector.x + y*Vector.y + z*Vector.z;
	}
	//	Cross Product
	const  CVector CVector::Cross(const CVector& Vector) const
	{	return CVector(
				y * Vector.z - Vector.y * z,
				z * Vector.x - Vector.z * x,
				x * Vector.y - Vector.x * y);
	}
	
	const bool CVector::operator > (const CVector& v)
	{
		return (x>v.x && y>v.y && z>v.z);
	}
	
	const bool CVector::operator < (const CVector& v)
	{
		return (x<v.x && y<v.y && z<v.z);
	}

	
	const float CVector::Magnitude() const
	{	return sqrtf(x*x + y*y + z*z);
	}
		
	//	Set length (Normalize if 1) or Magnitude of vector
	void CVector::Normalize()
	{
		float magnitude=this->Magnitude();
		*this/=magnitude;
	}


	const void CVector::SetNormal(CVector vTriangle0, CVector vTriangle1, CVector vTriangle2 )
	{
		*this = CVector((vTriangle2-vTriangle0)).Cross(vTriangle1-vTriangle0);				// Cross Product
		this->Normalize();						// Use our function we created to normalize the normal (Makes it a length of one)
	}

	//	The angle between two vectors in radians
	const float CVector::Angle(const CVector& Normal) const
	{	return acosf((*this).Dot(Normal));
	}
	//	Reflect in Normal Vector
	const CVector CVector::Reflection(const CVector& PlaneNormal) const
	{	return (*this - PlaneNormal * 2.0 * ((*this).Dot(PlaneNormal))) * (*this).Magnitude();
	}
	//	Rotate dAngle Degrees (radians) Around Normal
	const CVector CVector::Rotate(const float dAngle, const CVector& Normal) const
	{	const float dCos = cosf(dAngle);
		const float dSin = sinf(dAngle);
		return CVector(
				*this * dCos +
				((Normal * *this) * (1.0f - dCos)) * Normal +
				((*this).Dot(Normal)) * dSin);
	}


/* -------------------------------END VECTOR IMPLEMENTATION ------------------------------//
//----------------------------------------------------------------------------------------*/



/* --------------------------------- MATRIX IMPLEMENTATION -------------------------------//
//----------------------------------------------------------------------------------------*/

//---------------------------------------------------------------------------
//	MATRIX MULTIPLICATION OPERATORS
//---------------------------------------------------------------------------
void CMatrix::Multiply (CMatrix *matrix)
{
	float newMatrix[16];
	const float *m1 = m_matrix, *m2 = matrix->m_matrix;

	newMatrix[0] = m1[0]*m2[0] + m1[4]*m2[1] + m1[8]*m2[2];
	newMatrix[1] = m1[1]*m2[0] + m1[5]*m2[1] + m1[9]*m2[2];
	newMatrix[2] = m1[2]*m2[0] + m1[6]*m2[1] + m1[10]*m2[2];
	newMatrix[3] = 0;

	newMatrix[4] = m1[0]*m2[4] + m1[4]*m2[5] + m1[8]*m2[6];
	newMatrix[5] = m1[1]*m2[4] + m1[5]*m2[5] + m1[9]*m2[6];
	newMatrix[6] = m1[2]*m2[4] + m1[6]*m2[5] + m1[10]*m2[6];
	newMatrix[7] = 0;

	newMatrix[8] = m1[0]*m2[8] + m1[4]*m2[9] + m1[8]*m2[10];
	newMatrix[9] = m1[1]*m2[8] + m1[5]*m2[9] + m1[9]*m2[10];
	newMatrix[10] = m1[2]*m2[8] + m1[6]*m2[9] + m1[10]*m2[10];
	newMatrix[11] = 0;

	newMatrix[12] = m1[0]*m2[12] + m1[4]*m2[13] + m1[8]*m2[14] + m1[12];
	newMatrix[13] = m1[1]*m2[12] + m1[5]*m2[13] + m1[9]*m2[14] + m1[13];
	newMatrix[14] = m1[2]*m2[12] + m1[6]*m2[13] + m1[10]*m2[14] + m1[14];
	newMatrix[15] = 1;

	for (int i=0; i<16; i++)
		this->m_matrix[i] = newMatrix[i];
}


void CMatrix::SetTranslation(CVector pos)
{
	m_matrix[12] = pos.x;
	m_matrix[13] = pos.y;
	m_matrix[14] = pos.z;
	m_pos = pos;
}

void CMatrix::SetInvTranslation(CVector pos)
{
	m_matrix[12] = -pos.x;
	m_matrix[13] = -pos.y;
	m_matrix[14] = -pos.z;
	m_pos = pos;
}
	
void CMatrix::RotateDegrees(double dAngleX, double dAngleY, double dAngleZ)
{
	// 1 radian = 180 degrees/PI;
	float vec[3] = {0,0,0};

	vec[0] = ( float )( dAngleX*PI/180 );
	vec[1] = ( float )( dAngleY*PI/180 );
	vec[2] = ( float )( dAngleZ*PI/180 );

	RotateRadians( vec[0], vec[1], vec[2] );
}

void CMatrix::RotateInvDegrees(double dAngleX, double dAngleY, double dAngleZ)
{
	float vec[3];

	vec[0] = ( float )( dAngleX*PI/180 );
	vec[1] = ( float )( dAngleY*PI/180 );
	vec[2] = ( float )( dAngleZ*PI/180 );

	RotateInvRadians( vec[0], vec[1], vec[2] );
}


void CMatrix::RotateRadians(double dAngleX, double dAngleY, double dAngleZ)
{
	m_rot = CVector(dAngleX*180/PI, dAngleY*180/PI, dAngleZ*180/PI);

	double cr = FastCos1( dAngleX );
	double sr = FastSin1( dAngleX );
	double cp = FastCos1( dAngleY );
	double sp = FastSin1( dAngleY );
	double cy = FastCos1( dAngleZ );
	double sy = FastSin1( dAngleZ );

	m_matrix[0] = ( float )( cp*cy );
	m_matrix[1] = ( float )( cp*sy );
	m_matrix[2] = ( float )( -sp );

	double srsp = sr*sp;
	double crsp = cr*sp;

	m_matrix[4] = ( float )( srsp*cy-cr*sy );
	m_matrix[5] = ( float )( srsp*sy+cr*cy );
	m_matrix[6] = ( float )( sr*cp );

	m_matrix[8] = ( float )( crsp*cy+sr*sy );
	m_matrix[9] = ( float )( crsp*sy-sr*cy );
	m_matrix[10] = ( float )( cr*cp );
}


void CMatrix::RotateInvRadians(double dAngleX, double dAngleY, double dAngleZ)
{
	m_rot = CVector(dAngleX*180/PI, dAngleY*180/PI, dAngleZ*180/PI);

	double cr = cos( dAngleX );
	double sr = sin( dAngleX );
	double cp = cos( dAngleY );
	double sp = sin( dAngleY );
	double cy = cos( dAngleZ );
	double sy = sin( dAngleZ );

	// matrix0 = cos(y)*cos(z)
	//y = inv_cos(matrix0/cos(z))
	m_matrix[0] = ( float )( cp*cy );
	m_matrix[4] = ( float )( cp*sy );
	m_matrix[8] = ( float )( -sp );

	double srsp = sr*sp;
	double crsp = cr*sp;

	m_matrix[1] = ( float )( srsp*cy-cr*sy );
	m_matrix[5] = ( float )( srsp*sy+cr*cy );
	m_matrix[9] = ( float )( sr*cp );

	m_matrix[2] = ( float )( crsp*cy+sr*sy );
	m_matrix[6] = ( float )( crsp*sy-sr*cy );
	m_matrix[10] = ( float )( cr*cp );
}

//---------------------------------------------------------------------------

/* -------------------------------END MATRIX IMPLEMENTATION ------------------------------//
//----------------------------------------------------------------------------------------*/




/* ---------------------------- START QUATERNION IMPLEMENTATION --------------------------//
//----------------------------------------------------------------------------------------*/

void CQuaternion::CreateAxis(float x, float y, float z, float degree)
{
	
	float angle = float((degree / 180.0f) * PI);

	// Here we calculate the sin( theta / 2) once for optimization
	float result = (float)sin( angle / 2.0f );
		
	// Calcualte the w value by cos( sin( theta / 2 ) / 2 )
	w = (float)cos( angle / 2.0f );

	// Calculate the x, y and z of the quaternion
	x = float(x * result);
	y = float(y * result);
	z = float(z * result);

}

/*
GLvoid CQuaternion::AxisAngleToQuat(CVector axis, float theta)
{
        float halfTheta = theta * 0.5;
        float cosHalfTheta = cos(halfTheta);
        float sinHalfTheta = sin(halfTheta);
        x = axis.x * sinHalfTheta;
        y = axis.y * sinHalfTheta;
        z = axis.z * sinHalfTheta;
        w = cosHalfTheta;
}
*/
void CQuaternion::EulerToQuat(float fRoll, float fPitch, float fYaw)
{
	const float fSinPitch(sin(fPitch*0.5F));
	const float fCosPitch(cos(fPitch*0.5F));
	const float fSinYaw(sin(fYaw*0.5F));
	const float fCosYaw(cos(fYaw*0.5F));
	const float fSinRoll(sin(fRoll*0.5F));
	const float fCosRoll(cos(fRoll*0.5F));
	const float fCosPitchCosYaw(fCosPitch*fCosYaw);
	const float fSinPitchSinYaw(fSinPitch*fSinYaw);

	x = fSinRoll * fCosPitchCosYaw     - fCosRoll * fSinPitchSinYaw;
	y = fCosRoll * fSinPitch * fCosYaw + fSinRoll * fCosPitch * fSinYaw;
	z = fCosRoll * fCosPitch * fSinYaw - fSinRoll * fSinPitch * fCosYaw;
	w = fCosRoll * fCosPitchCosYaw     + fSinRoll * fSinPitchSinYaw;
}

void CQuaternion::Normalize()
{
  	float Mag;
  	Mag = GetMagnitude();
  	x = x/Mag;
  	y = y/Mag;
  	z = z/Mag;
	w = w/Mag;
}

float CQuaternion::GetMagnitude()
{
	return sqrtf(w*w + x*x + y*y + z*z);
}

CQuaternion CQuaternion::operator * (CQuaternion d)
{
	x = w*d.x + x*d.w + y*d.z - z*d.y;
	y = w*d.y + y*d.w + z*d.x - x*d.z;
	z = w*d.z + z*d.w + x*d.y - y*d.x;
	w = w*d.w - x*d.x - y*d.y - z*d.z;
	return *this;
}

void CQuaternion::Rotate(float degree, float X, float Y, float Z)
{
	float angle = float((degree / 180.0f) * PI);

	// Here we calculate the sin( theta / 2) once for optimization
	float result = (float)sin( angle / 2.0f );
		
	// Calcualte the w value by cos( sin( theta / 2 ) / 2 )
	w = (float)cos( angle / 2.0f );

	// Calculate the x, y and z of the quaternion
	x = float(X * result);
	y = float(Y * result);
	z = float(Z * result);
}

void CQuaternion::Derotate(float *angle, CVector axis)
{
	CQuaternion temp = *this;
	temp.Normalize();

	GLdouble cos_angle  = temp.w;
    *angle      = acosf( cos_angle )*2;
    GLdouble sin_angle  = sqrtf( 1.0 - cos_angle * cos_angle );
	GLdouble sa = 0.0;

    if ( fabs( sin_angle ) < 0.0005 )
      sa = 1;

    axis.x = temp.x / sa;
    axis.y = temp.y / sa;
    axis.z = temp.z / sa;
}

///////////////////////////////////////////////////////////////////////////////
// Function:	QuatToAxisAngle
// Purpose:		Convert a Quaternion to Axis Angle representation
// Arguments:	A quaternion to convert, a axisAngle to set
///////////////////////////////////////////////////////////////////////////////
CQuaternion CQuaternion::QuatToAxisAngle()
{
/// Local Variables ///////////////////////////////////////////////////////////
	float scale,tw;
	CQuaternion tmp;
///////////////////////////////////////////////////////////////////////////////
	tw = (float)acosf(w) * 2;
	scale = (float)sinf(tw / 2.0);
	tmp.x = x / scale;
	tmp.y = y / scale;
	tmp.z = z / scale;

	// NOW CONVERT THE ANGLE OF ROTATION BACK TO DEGREES
	tmp.w = (tw * (360 / 2)) / PI;

	return tmp;
}

CQuaternion CQuaternion::AxisAngleToQuat(CVector *axis, const float angle)
{
/// Local Variables ///////////////////////////////////////////////////////////
	float scale,tw;
	CQuaternion tmp;
///////////////////////////////////////////////////////////////////////////////
	tw = (angle * PI)/(180.0f);		// TO RADIANS
	tmp.w = cos(tw / 2.0f);
	scale = (float)sin(tw / 2.0);
	tmp.x = axis->x * scale;
	tmp.y = axis->y * scale;
	tmp.z = axis->z * scale;

	return tmp;
}

///////////////////////////////////////////////////////////////////////////////
// Function:	QuatToEuler
// Purpose:		Convert a Quaternion back to Euler Angles
// Arguments:	Quaternions and target Euler vector
// Notes:		The method is to convert Quaternion to a 3x3 matrix and
//				decompose the matrix.  This is subject to the
//				ambiguities of square roots and problems with inverse trig.
//				Matrix to Euler conversion is doublely very ill-defined but works
//				for my purposes.
///////////////////////////////////////////////////////////////////////////////
CVector CQuaternion::QuatToEuler(CQuaternion *quat)
{
/// Local Variables ///////////////////////////////////////////////////////////
	float matrix[3][3];
	float cx,sx;
	float cy,sy,yr;
	float cz,sz;
	CVector euler;
///////////////////////////////////////////////////////////////////////////////
	// CONVERT QUATERNION TO MATRIX - I DON'T doubleLY NEED ALL OF IT
	matrix[0][0] = 1.0f - (2.0f * quat->y * quat->y) - (2.0f * quat->z * quat->z);
//	matrix[0][1] = (2.0f * quat->x * quat->y) - (2.0f * quat->w * quat->z);
//	matrix[0][2] = (2.0f * quat->x * quat->z) + (2.0f * quat->w * quat->y);
	matrix[1][0] = (2.0f * quat->x * quat->y) + (2.0f * quat->w * quat->z);
//	matrix[1][1] = 1.0f - (2.0f * quat->x * quat->x) - (2.0f * quat->z * quat->z);
//	matrix[1][2] = (2.0f * quat->y * quat->z) - (2.0f * quat->w * quat->x);
	matrix[2][0] = (2.0f * quat->x * quat->z) - (2.0f * quat->w * quat->y);
	matrix[2][1] = (2.0f * quat->y * quat->z) + (2.0f * quat->w * quat->x);
	matrix[2][2] = 1.0f - (2.0f * quat->x * quat->x) - (2.0f * quat->y * quat->y);

	sy = -matrix[2][0];
	cy = sqrtf(1 - (sy * sy));
	yr = (float)atan2f(sy,cy);
	euler.y = (yr * 180.0f) / PI;

	// AVOID DIVIDE BY ZERO ERROR ONLY WHERE Y= +-90 or +-270 
	// NOT CHECKING cy BECAUSE OF PRECISION ERRORS
	if (sy != 1.0f && sy != -1.0f)	
	{
		cx = matrix[2][2] / cy;
		sx = matrix[2][1] / cy;
		euler.x = ((float)atan2f(sx,cx) * 180.0f) / PI;	// RAD TO DEG

		cz = matrix[0][0] / cy;
		sz = matrix[1][0] / cy;
		euler.z = ((float)atan2f(sz,cz) * 180.0f) / PI;	// RAD TO DEG
	}
	else
	{
		// SINCE Cos(Y) IS 0, I AM SCREWED.  ADOPT THE STANDARD Z = 0
		// I THINK THERE IS A WAY TO FIX THIS BUT I AM NOT SURE.  EULERS SUCK
		// NEED SOME MORE OF THE MATRIX TERMS NOW
		matrix[1][1] = 1.0f - (2.0f * quat->x * quat->x) - (2.0f * quat->z * quat->z);
		matrix[1][2] = (2.0f * quat->y * quat->z) - (2.0f * quat->w * quat->x);
		cx = matrix[1][1];
		sx = -matrix[1][2];
		euler.x = ((float)atan2f(sx,cx) * 180.0f) / PI;	// RAD TO DEG

		cz = 1.0f;
		sz = 0.0f;
		euler.z = ((float)atan2f(sz,cz) * 180.0f) / PI;	// RAD TO DEG
	}

	return euler;
}



///////////////////////////////////////////////////////////////////////////////
// Function:	SlerpQuat
// Purpose:		Spherical Linear Interpolation Between two Quaternions
// Arguments:	Two Quaternions, blend factor, result quaternion
// Notes:		The comments explain the handling of the special cases.
//				The comments in the magazine were wrong.  Adjust the
//				DELTA constant to play with the LERP vs. SLERP level
///////////////////////////////////////////////////////////////////////////////
//CQuaternion CQuaternion::SlerpQuat(tQuaternion *quat1,tQuaternion *quat2,float slerp, tQuaternion *result)
#define DELTA	0.0001		// DIFFERENCE AT WHICH TO LERP INSTEAD OF SLERP

CQuaternion CQuaternion::SlerpQuat(CQuaternion quat, float slerp)
{
/// Local Variables ///////////////////////////////////////////////////////////
	double omega,cosom,sinom,scale0,scale1;
	CQuaternion tmp;
///////////////////////////////////////////////////////////////////////////////
	// USE THE DOT PRODUCT TO GET THE COSINE OF THE ANGLE BETWEEN THE
	// QUATERNIONS
	cosom = x * quat.x + 
			y * quat.y + 
			z * quat.z + 
			w * quat.w; 

	// CHECK A COUPLE OF SPECIAL CASES. 
	// MAKE SURE THE TWO QUATERNIONS ARE NOT EXACTLY OPPOSITE? (WITHIN A LITTLE SLOP)
	if ((1.0 + cosom) > DELTA)
	{
		// ARE THEY MORE THAN A LITTLE BIT DIFFERENT? AVOID A DIVIDED BY ZERO AND LERP IF NOT
		if ((1.0 - cosom) > DELTA) {
			// YES, DO A SLERP
			omega = acos(cosom);
			sinom = sin(omega);
			scale0 = sin((1.0 - slerp) * omega) / sinom;
			scale1 = sin(slerp * omega) / sinom;
		} else {
			// NOT A VERY BIG DIFFERENCE, DO A LERP
			scale0 = 1.0 - slerp;
			scale1 = slerp;
		}
		tmp.x = scale0 * x + scale1 * quat.x;
		tmp.y = scale0 * y + scale1 * quat.y;
		tmp.z = scale0 * z + scale1 * quat.z;
		tmp.w = scale0 * w + scale1 * quat.w;
	} else {
		// THE QUATERNIONS ARE NEARLY OPPOSITE SO TO AVOID A DIVIDED BY ZERO ERROR
		// CALCULATE A PERPENDICULAR QUATERNION AND SLERP THAT DIRECTION
		tmp.x = -quat.y;
		tmp.y = quat.x;
		tmp.z = -quat.w;
		tmp.w = quat.z;
		scale0 = sin((1.0 - slerp) * (float)PI/2);
		scale1 = sin(slerp * (float)PI/2);
		tmp.x = scale0 * x + scale1 * tmp.x;
		tmp.y = scale0 * y + scale1 * tmp.y;
		tmp.z = scale0 * z + scale1 * tmp.z;
		tmp.w = scale0 * w + scale1 * tmp.w;
	}

	return tmp;
}
/* ---------------------------- END QUATERNION IMPLEMENTATION ----------------------------//
//----------------------------------------------------------------------------------------*/





// Function description - 
// calculates plane equation for OpenGL glClipPlane (and others) and puts them into passed
// variable 'arr', also returns normal to that plane.
CVector GetPlaneEq(float *arr, float *p1, float *p2, float *p3)
{
	//arr[0] is A, arr[1] is B, arr[2] is C, arr[3] is D
	
	/*
	A = y1 (z2 - z3) + y2 (z3 - z1) + y3 (z1 - z2) 
	B = z1 (x2 - x3) + z2 (x3 - x1) + z3 (x1 - x2) 
	C = x1 (y2 - y3) + x2 (y3 - y1) + x3 (y1 - y2) 
	- D = x1 (y2 z3 - y3 z2) + x2 (y3 z1 - y1 z3) + x3 (y1 z2 - y2 z1)
	*/
	if(arr !=NULL && p1 !=NULL && p2 !=NULL && p3 !=NULL)
	{
		arr[0] = p1[1] *(p2[2] - p3[2]) + p2[1] *(p3[2] - p1[2]) + p3[1] *(p1[2] - p2[2]);
		arr[1] = p1[2] *(p2[0] - p3[0]) + p2[2] *(p3[0] - p1[0]) + p3[2] *(p1[0] - p2[0]);
		arr[2] = p1[0] *(p2[1] - p3[1]) + p2[0] *(p3[1] - p1[1]) + p3[0] *(p1[1] - p2[1]);
	    arr[3] = -(p1[0] *(p2[1] * p3[2] - p3[1] * p2[2]) + p2[0] *(p3[1] * p1[2] - p1[1] * p3[2]) + p3[0] * (p1[1] * p2[2] - p2[1] * p1[2]));
	}

	CVector pt1(p1[0], p1[1], p1[2]), pt2(p2[0], p2[1], p2[2]), pt3(p3[0], p3[1], p3[2]);

	CVector normal = CVector(pt3 - pt1).Dot(pt2 - pt1);	// Take the cross product of our 2 vectors to get a perpendicular vector
	
	// normal.Normalize(); 

	return normal;

}

CVector GetPlaneEq(float *arr, CVector *p1, CVector *p2, CVector *p3)
{
	//arr[0] is A, arr[1] is B, arr[2] is C, arr[3] is D
	
	/*
	A = y1 (z2 - z3) + y2 (z3 - z1) + y3 (z1 - z2) 
	B = z1 (x2 - x3) + z2 (x3 - x1) + z3 (x1 - x2) 
	C = x1 (y2 - y3) + x2 (y3 - y1) + x3 (y1 - y2) 
	- D = x1 (y2 z3 - y3 z2) + x2 (y3 z1 - y1 z3) + x3 (y1 z2 - y2 z1)
	*/
	if(arr !=NULL && p1 !=NULL && p2 !=NULL && p3 !=NULL)
	{
		arr[0] = p1->y *(p2->z - p3->z) + p2->y *(p3->z - p1->z) + p3->y *(p1->z - p2->z); 
		arr[1] = p1->z *(p2->x - p3->x) + p2->z *(p3->x - p1->x) + p3->z *(p1->x - p2->x);
		arr[2] = p1->x *(p2->y - p3->y) + p2->x *(p3->y - p1->y) + p3->x *(p1->y - p2->y); 
	    arr[3] = -(p1->x *(p2->y * p3->z - p3->y * p2->z) + p2->x *(p3->y * p1->z - p1->y * p3->z) + p3->x * (p1->y * p2->z - p2->y * p1->z));
	}
	
	CVector normal = (p3 - p1)^(p2 - p1);	// Take the cross product of our 2 vectors to get a perpendicular vector
	// normal.Normalize(); 

	return normal;

}


CVector GetMousePos(int x, int y, int winy, int z, CVector *ret)
{
	int viewport[4];
	GLdouble mvmatrix[16], projmatrix[16], wx, wy, wz;

	glGetIntegerv (GL_VIEWPORT, viewport);
	glGetDoublev (GL_MODELVIEW_MATRIX, mvmatrix);
	glGetDoublev (GL_PROJECTION_MATRIX, projmatrix);
	
	/* note viewport[3] is height of window in pixels */
	unsigned int doubley = winy - (GLint) y - 1;
	
	gluUnProject ((GLdouble) x, (GLdouble) doubley, z,
	mvmatrix, projmatrix, viewport, &wx, &wy, &wz);

	if(ret !=NULL)
	{
		ret->x = wx;
		ret->y = wy;
		ret->z = wz;
		return CVector(0,0,0);
	}
	else
		return CVector(wx, wy, wz);
}

unsigned int FACT(unsigned int x)
{
	unsigned int Answer = 1;
	if(x == 0){return 1;}
	for (unsigned int i = 1;i<=x;i++)
	{Answer*= i;}
	return Answer;
}

int CombC( int N, int x )
{
	return FACT(N)/(FACT(x)*(FACT(N-x))) ;
}

void DrawArbitraryAxes(float axissize)
{
	glColor3d(1, 1, 1);

	// Draw X axis
	glBegin(GL_LINES);
	glVertex3f(0, 0, 0);
	glVertex3f(axissize, 0, 0);
	glEnd();

	//Draw Y axis
	glBegin(GL_LINES);
	glVertex3f(0, 0, 0);
	glVertex3f(0, axissize, 0);
	glEnd();

	// Draw Z axis
	glBegin(GL_LINES);
	glVertex3f(0, 0, 0);
	glVertex3f(0, 0, axissize);
	glEnd();
}



void QuaternionToMatrix(CQuaternion *quat, CMatrix *pMatrix)
{
	// Make sure the matrix has allocated memory to store the rotation data
	if(!pMatrix) return;
	/*	
         +        2     2                                      +
         + 1 - (2Y  + 2Z )   2XY + 2ZW         2XZ - 2YW       +
         +                                                     +
         +                          2     2                    +
     M = + 2XY - 2ZW         1 - (2X  + 2Z )   2YZ + 2XW       +
         +                                                     +
         +                                            2     2  +
         + 2XZ + 2YW         2YZ - 2XW         1 - (2X  + 2Y ) +
         +                                                     +
	*/

	GLdouble X = quat->x, Y = quat->y, Z = quat->z, W = quat->w;

	// First row
	pMatrix->m_matrix[0] = 1.0f - 2.0f * (X*Y - Z*W);  
	pMatrix->m_matrix[1] = 2.0f * (X*Y + Z*W);  
	pMatrix->m_matrix[2] = 2.0f * (X*Z - Y*W);  
	pMatrix->m_matrix[3] = 0.0f;  

	// Second row
	pMatrix->m_matrix[4] = 2.0f * (X*Y - Z*W);  
	pMatrix->m_matrix[5] = 1.0f - 2.0f * (X*X + Z*Z);  
	pMatrix->m_matrix[6] = 2.0f * (Y*Z + X*W);  
	pMatrix->m_matrix[7] = 0.0f;  

	// Third row
	pMatrix->m_matrix[8] = 2.0f * (X*Z + Y*W);  
	pMatrix->m_matrix[9] = 2.0f * (Y*Z - X*W);  
	pMatrix->m_matrix[10] = 1.0f - 2.0f * (X*X + Y*Y);  
	pMatrix->m_matrix[11] = 0.0f;  

	// Fourth row
	pMatrix->m_matrix[12] = 0;  
	pMatrix->m_matrix[13] = 0;  
	pMatrix->m_matrix[14] = 0;  
	pMatrix->m_matrix[15] = 1.0f;
}

/*
   0  1  2  3

0  0  1  2  3
1  4  5  6  7
2  8  9  10 11
3  12 13 14 15
*/
void MatrixToQuaternion(CQuaternion *quat, CMatrix *pMatrix)
{	
	
	GLdouble T = 1 + pMatrix->m_matrix[0] + pMatrix->m_matrix[5] + pMatrix->m_matrix[10];
	GLdouble S, X, Y, Z, W;

	if ( T > 0.00000001)
	{
		S = sqrt(T) * 2;
		X = ( pMatrix->m_matrix[9] - pMatrix->m_matrix[6] ) / S;
		Y = ( pMatrix->m_matrix[2] - pMatrix->m_matrix[8] ) / S;
		Z = ( pMatrix->m_matrix[4] - pMatrix->m_matrix[1] ) / S;
		W = 0.25 * S;
	}
	else
	{
		if ( pMatrix->m_matrix[0] > pMatrix->m_matrix[5] && pMatrix->m_matrix[0] > pMatrix->m_matrix[10] )  	// Column 0: 
		{
			S  = sqrt( 1.0 + pMatrix->m_matrix[0] - pMatrix->m_matrix[5] - pMatrix->m_matrix[10] ) * 2;
			X = 0.25 * S;
			Y = (pMatrix->m_matrix[4] + pMatrix->m_matrix[1] ) / S;
			Z = (pMatrix->m_matrix[2] + pMatrix->m_matrix[8] ) / S;
			W = (pMatrix->m_matrix[9] - pMatrix->m_matrix[6] ) / S;
		}
		else if ( pMatrix->m_matrix[5] > pMatrix->m_matrix[10] )// Column 1: 
		{			
			S  = sqrt( 1.0 + pMatrix->m_matrix[5] - pMatrix->m_matrix[0] - pMatrix->m_matrix[10] ) * 2;
			X = (pMatrix->m_matrix[4] + pMatrix->m_matrix[1] ) / S;
			Y = 0.25 * S;
			Z = (pMatrix->m_matrix[9] + pMatrix->m_matrix[6] ) / S;
			W = (pMatrix->m_matrix[2] - pMatrix->m_matrix[8] ) / S;
		}
		else 						// Column 2:
		{
			S  = sqrt( 1.0 + pMatrix->m_matrix[10] - pMatrix->m_matrix[0] - pMatrix->m_matrix[5] ) * 2;
			X = (pMatrix->m_matrix[2] + pMatrix->m_matrix[8] ) / S;
			Y = (pMatrix->m_matrix[9] + pMatrix->m_matrix[6] ) / S;
			Z = 0.25 * S;
			W = (pMatrix->m_matrix[4] - pMatrix->m_matrix[1] ) / S;
		}
	}

	quat->x = X;
	quat->y = Y;
	quat->z = Z;
	quat->w = W;
}


// Get distance from plane to point	 
float PlaneDistance(CVector Normal, CVector Point)
{
	return -((Normal.x * Point.x) + (Normal.y * Point.y) + (Normal.z * Point.z));
}

// Is Plane intersected?
bool IntersectedPlane(CVector vTriangle[], CVector vLine[])
{
		float distance1=0, distance2=0;						// The distances from the 2 points of the line from the plane
			
		CVector vNormal;
		vNormal.SetNormal(vTriangle[0], vTriangle[1], vTriangle[2]);				 // Normalize

		float originDistance = PlaneDistance(vNormal, vTriangle[0]);

	distance1 =	((vNormal.x * vLine[0].x)  +					// Ax +
		         (vNormal.y * vLine[0].y)  +					// Bx +
				 (vNormal.z * vLine[0].z)) + originDistance;	// Cz + D

	distance2 = ((vNormal.x * vLine[1].x)  +					// Ax +
		         (vNormal.y * vLine[1].y)  +					// Bx +
				 (vNormal.z * vLine[1].z)) + originDistance;	// Cz + D

	if(distance1 * distance2 >= 0)			// Check to see if both point's distances are both negative or both positive
	   return false;						// Return false if each point has the same sign.  -1 and 1 would mean each point is on either side of the plane.  -1 -2 or 3 4 wouldn't...
					
	return true;							// The line intersected the plane, Return TRUE

}





// RANDOM NUMBER GENERATOR****************************************************************************************************//
//****************************************************************************************************************************//
static	int	rgiState[2+55]; // leave this alone

int RANDOM(int min, int max)
{ return (rand()%(max-min+1))+min; }

int Random()
{
	int *piState;
    int iState1;
    int iState2;
    int iRand;
    piState		= &rgiState[2];
    iState1	 	= piState[-2];
    iState2	 	= piState[-1];
    iRand	 	= ( piState[iState1] + piState[iState2] )
    			& ( ( 1 << 30 ) - 1 );
    piState[iState1]	= iRand;
    
	if ( ++iState1 == 55 )
    	iState1 = 0;

    if ( ++iState2 == 55 )
    	iState2 = 0;

    piState[-2]		= iState1;
    piState[-1]		= iState2;

    return iRand >> 6;
}
    /*
    * Generate a random number.
    */
int Random(int from, int to)
{
    int power;
    int number;

    if ( ( to = to - from + 1 ) <= 1 )
    	return from;
    
	for ( power = 2; power < to; power <<= 1 )
    	;
    while ( ( number = Random( ) & ( power - 1 ) ) >= to )
    	;
    return from + number;
}


/*
* This is the Mitchell-Moore algorithm from Knuth Volume II. 
*/
void seed()
{
    int *piState;
    int iState;
    piState	= &rgiState[2];
    piState[-2]	= 55 - 55;
    piState[-1]	= 55 - 24;
    piState[0]	= ( (int) time( NULL ) ) & ( ( 1 << 30 ) - 1 );
    piState[1]	= 1;
    for ( iState = 2; iState < 55; iState++ )
    {
    	piState[iState] = ( piState[iState-1] + piState[iState-2] )
    			& ( ( 1 << 30 ) - 1 );
    }
    return;
}


// VARIOUS USEFUL FUNCTIONS
float DistancePointLine( CVector Point, CVector LineStart, CVector LineEnd)
{
    float LineMag;
    float U;
    CVector Intersection;
 
    LineMag = CVector(LineStart-LineEnd).Magnitude();
 
    U = ( ( ( Point.x - LineStart.x ) * ( LineEnd.x - LineStart.x ) ) +
        ( ( Point.y - LineStart.y ) * ( LineEnd.y - LineStart.y ) ) +
        ( ( Point.z - LineStart.z ) * ( LineEnd.z - LineStart.z ) ) ) /
        ( LineMag * LineMag );
 
    if( U < 0.0f || U > 1.0f )
        return 0;   // closest point does not fall within the line segment
 
    Intersection.x = LineStart.x + U * ( LineEnd.x - LineStart.x );
    Intersection.y = LineStart.y + U * ( LineEnd.y - LineStart.y );
    Intersection.z = LineStart.z + U * ( LineEnd.z - LineStart.z );
 
    return CVector(Point - Intersection).Magnitude();
}

void DrawSphere(CVector c, double r, int n, unsigned int render)
{
	glColor3d(1,1,1);
	int i,j;
   double theta1,theta2,theta3;
   CVector e,p;

   if (r < 0)
      r = -r;
   if (n < 0)
      n = -n;
   if (n < 4 || r <= 0) {
      glBegin(GL_POINTS);
      glVertex3f(c.x,c.y,c.z);
      glEnd();
      return;
   }

   for (j=0;j<n/2;j++) {
      theta1 = j * PI*2 / n -PI/2;
      theta2 = (j + 1) * PI*2 / n - PI/2 ;

      glBegin(render);
      for (i=0;i<=n;i++) {
         theta3 = i * 2*PI / n;

         e.x = cos(theta2) * cos(theta3);
         e.y = sin(theta2);
         e.z = cos(theta2) * sin(theta3);
         p.x = c.x + r * e.x;
         p.y = c.y + r * e.y;
         p.z = c.z + r * e.z;

         glNormal3f(e.x,e.y,e.z);
         glTexCoord2f(i/(double)n,2*(j+1)/(double)n);
         glVertex3f(p.x,p.y,p.z);

         e.x = cos(theta1) * cos(theta3);
         e.y = sin(theta1);
         e.z = cos(theta1) * sin(theta3);
         p.x = c.x + r * e.x;
         p.y = c.y + r * e.y;
         p.z = c.z + r * e.z;

         glNormal3f(e.x,e.y,e.z);
         glTexCoord2f(i/(double)n,2*j/(double)n);
         glVertex3f(p.x,p.y,p.z);
      }
      glEnd();
   }
}

bool SaveMatrix(CMatrix *mat, char *filename, FILE *f)
{
	if(mat == NULL)
		return false;

	FILE *file = NULL;
	
	if(f == NULL)
	{
		if(filename == NULL)
			return false;
		else
			file = fopen(filename,"w+t");
	}
	else
		file = f;
	
	if(file !=NULL)
	{
		fprintf(file, "%f %f %f %f %f %f %f %f %f\n", 
						mat->m_pos.x, mat->m_pos.y, mat->m_pos.z,
						mat->m_rot.x, mat->m_rot.y, mat->m_rot.z,
						mat->m_scale.x, mat->m_scale.y, mat->m_scale.z);
	}
	
	if(f == NULL)
		fclose(file);

	return true;
}


bool LoadMatrix(CMatrix *mat, char *filename, FILE *f)
{

	if(mat == NULL)
		return false;

	FILE *file = NULL;
	
	if(f == NULL)
	{
		if(filename == NULL)
			return false;
		else
			file = fopen(filename,"r+t");
	}
	else
		file = f;
	
	if(file !=NULL)
	{
		fscanf(file, "%f %f %f %f %f %f %f %f %f\n", 
						&mat->m_pos.x, &mat->m_pos.y, &mat->m_pos.z,
						&mat->m_rot.x, &mat->m_rot.y, &mat->m_rot.z,
						&mat->m_scale.x, &mat->m_scale.y, &mat->m_scale.z);

		mat->SetTranslation(mat->m_pos);
		mat->RotateDegrees(mat->m_rot.x, mat->m_rot.y, mat->m_rot.z);
		mat->Scale(mat->m_scale.x, mat->m_scale.y, mat->m_scale.z);

		//fgetc(file);	// SKIP NEWLINE
	}

	if(f == NULL)
		fclose(file);

	return true;
}
	