// Model.h: interface for the CModel class.
//
//////////////////////////////////////////////////////////////////////
/*
	Model.h

		Abstract base class for a model. The specific extended class will render the given model. 

	Author:	Brett Porter
	Email: brettporter@yahoo.com
	Website: http://www.geocities.com/brettporter/
	Copyright (C)2000, Brett Porter. All Rights Reserved.

	This file may be used only as long as this copyright notice remains intact.
*/

#include "Material.h"
#include "Maths.h"
#include "Timer.h"

#if !defined(AFX_MODEL_H__5B1748E7_9FE3_4826_B72B_82B2325F7ACF__INCLUDED_)
#define AFX_MODEL_H__5B1748E7_9FE3_4826_B72B_82B2325F7ACF__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CModel
{
public:
	char m_strFilename[255];
		
	unsigned int m_ID;
	unsigned int m_iListID;

	CMatrix m_ViewMatrix;
	char m_strName[64];

	MAT_RENDER m_eMaterialRender;
	unsigned int m_iVertexRender;
	RenderMode m_eRM;
	

	unsigned int m_iNumMeshes;
	tMesh *m_pMeshes;

	//	Materials used
	int m_iNumMaterials;
	tMaterial *m_pMaterials;
	
	//	Triangles used
	int m_iNumTriangles;
	tTriangle *m_pTriangles;

	//	Vertices Used
	int m_iNumVertices;
	tBoneVertex *m_pVertices;

	DWORD m_wReserved;

	int m_iNumJoints;
	tJoint *m_pJoints;

	//	Timer variable
	CTimer m_Timer;

	//	Total animation time
	double m_fTotalTime;

	//	Is the animation looping?
	bool m_bLooping;

	// END NEW

	void Draw();
	void calculateNormals();
	void Destroy_Model();

	// NEW
		/*	
			Set the values of a particular keyframe for a particular joint.
				jointIndex		The joint to setup the keyframe for
				keyframeIndex	The maximum number of keyframes
				time			The time in milliseconds of the keyframe
				parameter		The rotation/translation values for the keyframe
				isRotation		Whether it is a rotation or a translation keyframe
		*/
	void SetJointKeyframe( int jointIndex, int keyframeIndex, float time, float *parameter, bool isRotation );

	//	Setup joint matrices.
		void SetupJoints();

	//	Advance animation by a frame. 
		void Animate();

	//	Restart animation.
		void Restart();
	// END NEW 


	virtual bool Load_Model(char *filename, bool bAnimated) { return false;};
	
	CModel();
	~CModel() {Destroy_Model();};

};

#endif // !defined(AFX_MODEL_H__5B1748E7_9FE3_4826_B72B_82B2325F7ACF__INCLUDED_)
